@@ -29,6 +29,7 @@
 #include <QTimer>
 #include <QFile>
 #include <QUuid>
+#include <QPainter>
 #include <QDBusPendingReply>
 #include "installerprompt.h"
 #include "wifipassworddialog.h"
@@ -44,8 +45,6 @@ InstallerPrompt::InstallerPrompt(QWidget *parent)
 
     cpd = new ConnectionProgressDialog();
 
-    updateConnectionInfo();
-
     initLanguageComboBox();
 
     connect(ui->tryKubuntu, &QPushButton::clicked, this, &InstallerPrompt::onTryClicked);
@@ -65,6 +64,8 @@ InstallerPrompt::InstallerPrompt(QWidget *parent)
     QTimer *repeater = new QTimer();
     connect(repeater, SIGNAL(timeout()), this, SLOT(updateConnectionInfo()));
     repeater->start(15000);
+    
+    updateConnectionInfo();
 }
 
 InstallerPrompt::~InstallerPrompt()
@@ -75,28 +76,25 @@ InstallerPrompt::~InstallerPrompt()
 void InstallerPrompt::activateBackground()
 {
     // Set the background image and scale it
-    QImage image(":/background");
-    if (image.isNull()) {
+    QImage rawImage(":/background");
+    if (rawImage.isNull()) {
         WarningDialog::showWarning(tr("Background image cannot be loaded."));
         return;
     }
 
-    qreal imgRatio = static_cast<qreal>(image.width()) / image.height();
+    qreal imgRatio = static_cast<qreal>(rawImage.width()) / rawImage.height();
     qreal screenRatio = static_cast<qreal>(this->width()) / this->height();
     QImage scaled;
     if (imgRatio < screenRatio) {
-        scaled = image.scaledToWidth(this->width(), Qt::SmoothTransformation);
+        scaled = rawImage.scaledToWidth(this->width(), Qt::SmoothTransformation);
         int yGap = (scaled.height() - this->height()) / 2;
         scaled = scaled.copy(0, yGap, scaled.width(), this->height());
     } else {
-        scaled = image.scaledToHeight(this->height(), Qt::SmoothTransformation);
+        scaled = rawImage.scaledToHeight(this->height(), Qt::SmoothTransformation);
         int xGap = (scaled.width() - this->width()) / 2;
         scaled = scaled.copy(xGap, 0, this->width(), scaled.height());
     }
-    QPixmap bg = QPixmap::fromImage(scaled);
-    QPalette palette;
-    palette.setBrush(QPalette::Window, bg);
-    this->setPalette(palette);
+    background = scaled;
 }
 
 void InstallerPrompt::onTryClicked()
@@ -175,7 +173,7 @@ void InstallerPrompt::onNetworkSelected(int index)
     QString networkId = ui->networkComboBox->itemData(index).toString();
     if (!networkId.isNull()) {
         if (networkId.left(4) == "UNI_") { // this is an Ethernet device
-            QString deviceUni = networkId.right(networkId.count() - 4);
+            QString deviceUni = networkId.right(networkId.length() - 4);
             NetworkManager::WiredDevice wiredDevice(deviceUni);
             QDBusPendingReply reply = wiredDevice.disconnectInterface();
             reply.waitForFinished();
@@ -190,9 +188,21 @@ void InstallerPrompt::onNetworkSelected(int index)
             }
         } else { // this is a Wifi connection
             wifiSsid = networkId.right(networkId.length() - 4);
+            bool isPasswordProtected = true;
+
+            for (const auto &network : wifiDevice->networks()) {
+                if (network->ssid() == wifiSsid) {
+                    NetworkManager::AccessPoint::Ptr ap = network->referenceAccessPoint();
+                    if (!ap->capabilities().testFlag(NetworkManager::AccessPoint::Privacy)) {
+                        isPasswordProtected = false;
+                    }
+                }
+            }
+
             QDBusPendingReply reply = wifiDevice->disconnectInterface();
             reply.waitForFinished();
             NetworkManager::Connection::Ptr targetConnection;
+
             foreach (const NetworkManager::Connection::Ptr &connection, NetworkManager::listConnections()) {
                 if (connection->settings()->connectionType() == NetworkManager::ConnectionSettings::Wireless) {
                     auto wirelessSetting = connection->settings()->setting(NetworkManager::Setting::Wireless).dynamicCast<NetworkManager::WirelessSetting>();
@@ -201,17 +211,11 @@ void InstallerPrompt::onNetworkSelected(int index)
                     }
                 }
             }
+
             if (targetConnection) {
                 NetworkManager::activateConnection(targetConnection->path(), wifiDevice->uni(), QString());
                 cpd->setNetworkName(wifiSsid);
             } else {
-                WifiPasswordDialog wpd(wifiSsid);
-                wpd.exec();
-                QString password = wpd.getPassword();
-                if (password.isEmpty()) {
-                    return;
-                }
-
                 NMVariantMapMap wifiSettings;
                 if (!wifiDevice) {
                     qWarning() << "WiFi device not found. Unable to set interface name.";
@@ -230,10 +234,18 @@ void InstallerPrompt::onNetworkSelected(int index)
                     wifiSettings.insert(key, value.toMap());
                 }
 
-                QVariantMap wirelessSecurity;
-                wirelessSecurity["key-mgmt"] = "wpa-psk";
-                wirelessSecurity["psk"] = password;
-                wifiSettings["802-11-wireless-security"] = wirelessSecurity;
+                if (isPasswordProtected) {
+                    WifiPasswordDialog wpd(wifiSsid);
+                    wpd.exec();
+                    QString password = wpd.getPassword();
+                    if (password.isEmpty()) {
+                        return;
+                    }
+                    QVariantMap wirelessSecurity;
+                    wirelessSecurity["key-mgmt"] = "wpa-psk";
+                    wirelessSecurity["psk"] = password;
+                    wifiSettings["802-11-wireless-security"] = wirelessSecurity;
+                }
 
                 QDBusPendingReply<QDBusObjectPath> reply = NetworkManager::addConnection(wifiSettings);
                 reply.waitForFinished();
@@ -303,10 +315,14 @@ void InstallerPrompt::updateConnectionInfo()
         ui->networkComboBox->setCurrentIndex(connectedDevice);
     }
 
-    if (NetworkManager::status() == NetworkManager::Connecting) {
-        cpd->exec();
+    if (!firstUpdateConnectionInfoCall) {
+        if (NetworkManager::status() == NetworkManager::Connecting) {
+            cpd->exec();
+        } else {
+            cpd->done(0);
+        }
     } else {
-        cpd->done(0);
+        firstUpdateConnectionInfoCall = false;
     }
 }
 
@@ -412,9 +428,9 @@ QString InstallerPrompt::getDisplayNameForLocale(const QLocale &locale) {
 
     QLocale currentAppLocale = QLocale::system();
     QString nativeName = locale.nativeLanguageName();
-    QString nativeCountryName = sanitize(locale.nativeCountryName());
+    QString nativeCountryName = sanitize(locale.nativeTerritoryName());
     QString englishLanguageName = currentAppLocale.languageToString(locale.language());
-    QString englishCountryName = sanitize(currentAppLocale.countryToString(locale.country()));
+    QString englishCountryName = sanitize(currentAppLocale.territoryToString(locale.territory()));
 
     if (nativeName.isEmpty() || nativeCountryName.isEmpty()) {
         return QString();
@@ -434,3 +450,8 @@ QString InstallerPrompt::getDisplayNameForLocale(const QLocale &locale) {
 
     return displayName;
 }
+
+void InstallerPrompt::paintEvent(QPaintEvent *event) {
+    QPainter painter(this);
+    painter.drawImage(0, 0, background);
+}
